@@ -352,7 +352,7 @@ mod tests {
     use crate::client::client_api::blob_storage::BlobStorage;
     use crate::client::utils::{
         generate_random_vector,
-        test_utils::{create_test_client, run_w_backoff},
+        test_utils::{create_test_client, run_w_backoff_delayed},
     };
     use crate::retry_err_loop;
     use crate::types::{PrivateChunk, PublicChunk};
@@ -362,6 +362,8 @@ mod tests {
     use self_encryption::Storage;
     use tokio::time::{Duration, Instant};
 
+    const BLOB_TEST_QUERY_TIMEOUT: u64 = 60;
+
     // Test storing and getting public Blob.
     #[tokio::test]
     #[ignore = "too heavy for CI"]
@@ -418,13 +420,6 @@ mod tests {
         let value = generate_random_vector::<u8>(10);
         let (_, expected_address) = Client::blob_data_map(value.clone(), None).await?;
 
-        // Get non-existent Blob
-        match client.read_blob(expected_address, None, None).await {
-            Ok(data) => bail!("public chunk should not exist yet: {:?}", data),
-            Err(Error::NoResponse) => (),
-            Err(e) => bail!("Unexpected: {:?}", e),
-        }
-
         // Store blob
         let public_address = client.store_public_blob(&value).await?;
         // check it's the expected Blob address
@@ -432,7 +427,7 @@ mod tests {
 
         // Assert that the blob was written
         let fetched_data =
-            run_w_backoff(|| client.read_blob(public_address, None, None), 10).await?;
+            run_w_backoff_delayed(|| client.read_blob(public_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         // Test storing public chunk with the same value.
@@ -453,20 +448,14 @@ mod tests {
         let owner = client.public_key();
         let (_, expected_address) = Client::blob_data_map(value.clone(), Some(owner)).await?;
 
-        // Get non-existent Blob
-        match client.read_blob(expected_address, None, None).await {
-            Ok(_) => bail!("private chunk should not exist yet"),
-            Err(Error::NoResponse) => (),
-            Err(e) => bail!("Expecting NoResponse error, got: {:?}", e),
-        }
-
         // Store Blob
         let priv_address = client.store_private_blob(&value).await?;
         // check it's the expected Blob address
         assert_eq!(expected_address, priv_address);
 
         // Assert that the blob is stored.
-        let fetched_data = run_w_backoff(|| client.read_blob(priv_address, None, None), 10).await?;
+        let fetched_data =
+            run_w_backoff_delayed(|| client.read_blob(priv_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         // Test storing private chunk with the same value.
@@ -479,7 +468,7 @@ mod tests {
 
         // Assert that the public Blob is stored.
         let fetched_data =
-            run_w_backoff(|| client.read_blob(public_address, None, None), 10).await?;
+            run_w_backoff_delayed(|| client.read_blob(public_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         // Delete Blob
@@ -508,7 +497,8 @@ mod tests {
         assert_eq!(new_addr, priv_address);
 
         // Assert that the Blob is stored again.
-        let fetched_data = run_w_backoff(|| client.read_blob(priv_address, None, None), 10).await?;
+        let fetched_data =
+            run_w_backoff_delayed(|| client.read_blob(priv_address, None, None), 10).await?;
         assert_eq!(value, fetched_data);
 
         Ok(())
@@ -522,10 +512,10 @@ mod tests {
         let address = client.store_private_blob(&value).await?;
 
         // let's make sure we have all chunks stored on the network
-        let _ = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
+        let _ = run_w_backoff_delayed(|| client.read_blob(address, None, None), 10).await?;
 
         let fetched_data =
-            run_w_backoff(|| client.fetch_blob_from_network(address, false), 10).await?;
+            run_w_backoff_delayed(|| client.fetch_blob_from_network(address, false), 10).await?;
 
         let root_data_map = match deserialize(fetched_data.value())? {
             DataMapLevel::Root(data_map) => data_map,
@@ -536,7 +526,6 @@ mod tests {
         };
 
         client.delete_blob(address).await?;
-
         client.clear_blob_cache().await;
 
         client.query_timeout = Duration::from_secs(5); // override with a short timeout
@@ -546,7 +535,6 @@ mod tests {
             for chunk in chunks {
                 let _ = retry_err_loop!(blob_storage.get(&chunk.hash));
             }
-
             Ok(())
         } else {
             Err(anyhow!(
@@ -560,16 +548,13 @@ mod tests {
     #[tokio::test]
     pub async fn create_and_retrieve_1kb_pub_unencrypted() -> Result<()> {
         let size = 1024;
-
         gen_data_then_create_and_retrieve(size, true).await?;
-
         Ok(())
     }
 
     #[tokio::test]
     pub async fn create_and_retrieve_1kb_private_unencrypted() -> Result<()> {
         let size = 1024;
-
         gen_data_then_create_and_retrieve(size, false).await?;
         Ok(())
     }
@@ -583,7 +568,7 @@ mod tests {
         let address = client.store_public_blob(&data).await?;
 
         // let's make sure the public chunk is stored
-        let _ = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
+        let _ = run_w_backoff_delayed(|| client.read_blob(address, None, None), 10).await?;
 
         // and now trying to read a private chunk with same address should fail
         let res = client
@@ -605,9 +590,9 @@ mod tests {
         let address = client.store_private_blob(&value).await?;
 
         // let's make sure the private chunk is stored
-        let _ = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
+        let _ = run_w_backoff_delayed(|| client.read_blob(address, None, None), 10).await?;
 
-        // and now trying to read a public chunk with same address should fail
+        // and now trying to read a public chunk with same address should fail (timeout)
         let res = client
             .read_blob(ChunkAddress::Public(*address.name()), None, None)
             .await;
@@ -637,7 +622,6 @@ mod tests {
     pub async fn create_and_retrieve_10mb_private() -> Result<()> {
         let size = 1024 * 1024 * 10;
         gen_data_then_create_and_retrieve(size, false).await?;
-
         Ok(())
     }
 
@@ -653,12 +637,9 @@ mod tests {
     pub async fn create_and_retrieve_100mb_public() -> Result<()> {
         let size = 1024 * 1024 * 100;
         gen_data_then_create_and_retrieve(size, true).await?;
-
         Ok(())
     }
 
-    const BLOB_TEST_QUERY_TIMEOUT: u64 = 60;
-
     #[tokio::test]
     pub async fn create_and_retrieve_index_based() -> Result<()> {
         create_and_index_based_retrieve(1024).await
@@ -672,7 +653,7 @@ mod tests {
         let address = client.store_public_blob(&data).await?;
 
         let fetched_data =
-            run_w_backoff(|| client.read_blob(address, None, Some(size / 2)), 10).await?;
+            run_w_backoff_delayed(|| client.read_blob(address, None, Some(size / 2)), 10).await?;
         assert_eq!(fetched_data, data[0..size / 2].to_vec());
 
         // Test read second half
@@ -681,7 +662,7 @@ mod tests {
 
         let address = client.store_public_blob(&data).await?;
 
-        let fetched_data = run_w_backoff(
+        let fetched_data = run_w_backoff_delayed(
             || client.read_blob(address, Some(size / 2), Some(size / 2)),
             10,
         )
@@ -706,7 +687,7 @@ mod tests {
 
         let address_before = chunk.address();
 
-        // attempt to retrieve it with generated address (it should error)
+        // attempt to retrieve it with generated address (it should error, i.e. timeout)
         let res = client.read_blob(*address_before, None, None).await;
         match res {
             Err(Error::NoResponse) => (),
@@ -723,7 +704,8 @@ mod tests {
         };
 
         // now that it was put to the network we should be able to retrieve it
-        let fetched_data = run_w_backoff(|| client.read_blob(address, None, None), 10).await?;
+        let fetched_data =
+            run_w_backoff_delayed(|| client.read_blob(address, None, None), 10).await?;
         // then the content should be what we put
         assert_eq!(fetched_data, raw_data);
 
