@@ -10,8 +10,8 @@ use super::Client;
 use crate::client::Error;
 use crate::messaging::client::{ClientSig, Cmd};
 use crate::types::{PublicKey, Signature};
-use log::debug;
 use std::net::SocketAddr;
+use tracing::debug;
 
 impl Client {
     /// Send a signed Cmd to the network
