@@ -83,18 +83,11 @@ impl Client {
     /// You're only able to delete a PrivateRegister. Public data can no be removed from the network.
     pub async fn delete_register(&self, address: Address) -> Result<(), Error> {
         let cmd = DataCmd::Register(RegisterWrite::Delete(address));
-        // Payment for PUT
-        let payment_proof = self.create_write_payment_proof(&cmd).await?;
 
         // The _actual_ message
-        let cmd = Cmd::Data {
-            cmd,
-            payment: payment_proof.clone(),
-        };
+        let cmd = Cmd::Data { cmd };
 
-        self.send_cmd(cmd, None).await?;
-
-        self.apply_write_payment_to_local_actor(payment_proof).await
+        self.send_cmd(cmd, None).await
     }
 
     /// Write to Register
@@ -239,23 +232,60 @@ impl Client {
 #[cfg(test)]
 mod tests {
     use crate::client::{
-        utils::test_utils::{create_test_client, gen_ed_keypair},
+        utils::test_utils::{create_test_client, gen_ed_keypair, run_w_backoff},
         Error,
     };
     use crate::messaging::client::Error as ErrorMessage;
+    use crate::retry_loop_for_pattern;
     use crate::types::{
         register::{Action, EntryHash, Permissions, PrivatePermissions, PublicPermissions, User},
-        Error as DtError, PublicKey, Token,
+        Error as DtError, PublicKey,
     };
-    use crate::{retry_loop, retry_loop_for_pattern};
     use anyhow::{anyhow, bail, Result};
     use std::{
         collections::{BTreeMap, BTreeSet},
-        str::FromStr,
+        time::Instant,
     };
     use tokio::time::Duration;
     use xor_name::XorName;
 
+    #[tokio::test]
+    #[ignore = "too heavy for CI"]
+    pub async fn measure_upload_times() -> Result<()> {
+        let mut total = 0;
+
+        let name = XorName(rand::random());
+        let tag = 10;
+        let client = create_test_client(None).await?;
+
+        let owner = client.public_key();
+        let mut perms = BTreeMap::<User, PublicPermissions>::new();
+        let _ = perms.insert(User::Key(owner), PublicPermissions::new(true));
+
+        let address = client
+            .store_public_register(name, tag, owner, perms)
+            .await?;
+
+        for i in 0..1000_usize {
+            let now = Instant::now();
+
+            // write to the register
+            let _value1_hash = run_w_backoff(
+                || client.write_to_register(address, b"VALUE1".to_vec(), BTreeSet::new()),
+                10,
+            )
+            .await?;
+
+            let elapsed = now.elapsed().as_millis();
+            total += elapsed;
+            println!("Iter # {}, elapsed: {}", i, elapsed);
+        }
+
+        println!("Total elapsed: {}", total);
+
+        Ok(())
+    }
+
     /**** Register data tests ****/
 
     #[tokio::test]
@@ -273,7 +303,7 @@ mod tests {
             .store_private_register(name, tag, owner, perms)
             .await?;
 
-        let register = retry_loop!(client.get_register(address));
+        let register = run_w_backoff(|| client.get_register(address), 10).await?;
 
         assert!(register.is_private());
         assert_eq!(*register.name(), name);
@@ -288,7 +318,7 @@ mod tests {
             .store_public_register(name, tag, owner, perms)
             .await?;
 
-        let register = retry_loop!(client.get_register(address));
+        let register = run_w_backoff(|| client.get_register(address), 10).await?;
 
         assert!(register.is_public());
         assert_eq!(*register.name(), name);
@@ -311,11 +341,15 @@ mod tests {
             .store_private_register(name, tag, owner, perms)
             .await?;
 
-        let register = retry_loop!(client.get_register(address));
+        let register = run_w_backoff(|| client.get_register(address), 10).await?;
 
         assert_eq!(register.size(None)?, 0);
 
-        let permissions = retry_loop!(client.get_register_permissions_for_user(address, owner));
+        let permissions = run_w_backoff(
+            || client.get_register_permissions_for_user(address, owner),
+            10,
+        )
+        .await?;
 
         match permissions {
             Permissions::Private(user_perms) => {
@@ -349,7 +383,11 @@ mod tests {
             .store_public_register(name, tag, owner, perms)
             .await?;
 
-        let permissions = retry_loop!(client.get_register_permissions_for_user(address, owner));
+        let permissions = run_w_backoff(
+            || client.get_register_permissions_for_user(address, owner),
+            10,
+        )
+        .await?;
 
         match permissions {
             Permissions::Public(user_perms) => {
@@ -387,8 +425,11 @@ mod tests {
             .await?;
 
         // write to the register
-        let value1_hash =
-            retry_loop!(client.write_to_register(address, b"VALUE1".to_vec(), BTreeSet::new()));
+        let value1_hash = run_w_backoff(
+            || client.write_to_register(address, b"VALUE1".to_vec(), BTreeSet::new()),
+            10,
+        )
+        .await?;
 
         // now check last entry
         let hashes = retry_loop_for_pattern!(client.read_register(address), Ok(hashes) if !hashes.is_empty())?;
@@ -398,8 +439,11 @@ mod tests {
         assert_eq!(current, Some(&(value1_hash, b"VALUE1".to_vec())));
 
         // write to the register
-        let value2_hash =
-            retry_loop!(client.write_to_register(address, b"VALUE2".to_vec(), BTreeSet::new()));
+        let value2_hash = run_w_backoff(
+            || client.write_to_register(address, b"VALUE2".to_vec(), BTreeSet::new()),
+            10,
+        )
+        .await?;
 
         // and then lets check last entry
         let hashes =
@@ -442,7 +486,7 @@ mod tests {
             .await?;
 
         // Assert that the data is stored.
-        let current_owner = retry_loop!(client.get_register_owner(address));
+        let current_owner = run_w_backoff(|| client.get_register_owner(address), 10).await?;
 
         assert_eq!(owner, current_owner);
 
@@ -463,7 +507,7 @@ mod tests {
             .store_private_register(name, tag, owner, perms)
             .await?;
 
-        let register = retry_loop!(client.get_register(address));
+        let register = run_w_backoff(|| client.get_register(address), 10).await?;
 
         assert!(register.is_private());
 
@@ -503,7 +547,7 @@ mod tests {
             .store_public_register(name, tag, owner, perms)
             .await?;
 
-        let register = retry_loop!(client.get_register(address));
+        let register = run_w_backoff(|| client.get_register(address), 10).await?;
         assert!(register.is_public());
 
         match client.delete_register(address).await {
@@ -524,27 +568,4 @@ mod tests {
 
         Ok(())
     }
-
-    #[tokio::test]
-    pub async fn register_deletions_should_cost_put_price() -> Result<()> {
-        let name = XorName(rand::random());
-        let tag = 10;
-        let client = create_test_client(None).await?;
-        let owner = client.public_key();
-        let perms = BTreeMap::<PublicKey, PrivatePermissions>::new();
-        let address = client
-            .store_private_register(name, tag, owner, perms)
-            .await?;
-
-        let balance_before_delete = retry_loop_for_pattern!(client.get_balance(),
-            Ok(balance) if *balance != Token::from_str("0")? && *balance != Token::from_str("10")?)?;
-
-        client.delete_register(address).await?;
-
-        // now let's ensure we've paid _something_
-        let _ = retry_loop_for_pattern!(client.get_balance(),
-            Ok(balance) if *balance != balance_before_delete)?;
-
-        Ok(())
-    }
 }
