@@ -8,11 +8,10 @@
 
 pub use crate::messaging::{client::Error as ErrorMessage, Error as MessagingError};
 use crate::messaging::{
-    client::{CmdError, Event, QueryResponse, TransferError},
+    client::{CmdError, QueryResponse},
     MessageId,
 };
-pub use crate::transfers::Error as TransfersError;
-use crate::types::{Error as DtError, PublicKey};
+use crate::types::Error as DtError;
 use qp2p::Error as QuicP2pError;
 use std::io;
 
@@ -67,12 +66,6 @@ pub enum Error {
     /// No section prefix found for session
     #[error("We do not have a section prefix.")]
     NoSectionPrefixKnown,
-    /// No transfer validation listener .
-    #[error("No transfer validation listener")]
-    NoTransferValidationListener,
-    /// Received transfer history was behind locally known history. Update commands have been sent to elders. Retry the operation shortly/
-    #[error("Received transfer history was behind locally known history. Update commands have been sent to elders. Retry the operation shortly/")]
-    ElderHistoryOutofDate,
     /// Unexpected message type receivied while joining.
     #[error("Unexpected message type receivied while joining: {0}")]
     UnexpectedMessageOnJoin(String),
@@ -86,38 +79,17 @@ pub enum Error {
     #[error("Could not listen on elder connection")]
     NoElderListenerEstablished,
     /// Incorrect user permissions were returned
-    #[error(" Incorrect user permissions were returned")]
+    #[error("Incorrect user permissions were returned")]
     IncorrectPermissions,
-    /// Unexpcted transfer event received
-    #[error("Unexpcted transfer event received {0:?}")]
-    UnexpectedTransferEvent(Event),
-    /// Unexpcted response received
+    /// Unexpected response received
     #[error("Unexpected response received when querying {0:?}")]
     UnexpectedQueryResponse(QueryResponse),
-    /// Unexpected response received
-    #[error("Unexpected response received when querying balance history {0:?}")]
-    UnexpectedHistoryResponse(QueryResponse),
-    /// Unexpected response received
-    #[error("Unexpected response received when querying store cost: {0:?}")]
-    UnexpectedStoreCostResponse(QueryResponse),
-    /// Unexpected response received
-    #[error("Unexpected response received when querying replica keys for PublicKey: {0:?}")]
-    UnexpectedReplicaKeysResponse(PublicKey),
-    /// Transfer actor failed generating a transfer
-    #[error("No transfer generated by transfer actor")]
-    NoTransferGenerated,
-    /// Transfer actor did not find any events to register locally
-    #[error("Transfer actor did not find any events to register locally")]
-    NoTransferEventsForLocalActor,
     /// Not in testnet "simulated payout" mode
     #[error("Simulated payouts unavailable without 'simualted-payouts' feature flag at build")]
     NotBuiltWithSimulatedPayouts,
     /// Other types errors
     #[error(transparent)]
     NetworkDataError(#[from] DtError),
-    /// Transfers errors
-    #[error(transparent)]
-    Transfer(#[from] TransfersError),
     /// Errors received from the network via sn_messaging
     #[error(
         "Error received from the network: {:?} MessageId: {:?}",
@@ -152,13 +124,7 @@ pub enum Error {
 
 impl From<(CmdError, MessageId)> for Error {
     fn from((error, msg_id): (CmdError, MessageId)) -> Self {
-        let source = match error {
-            CmdError::Data(data_err) => data_err,
-            CmdError::Transfer(err) => match err {
-                TransferError::TransferValidation(err) => err,
-                TransferError::TransferRegistration(err) => err,
-            },
-        };
+        let CmdError::Data(source) = error;
         Error::ErrorMessage { source, msg_id }
     }
 }
