@@ -407,7 +407,7 @@ fn to_id(address: &Address) -> Result<XorName> {
 }
 
 fn new_store(id: XorName, path: &Path) -> Result<EventStore<SequenceCmd>> {
-    let db_dir = path.join("db").join("map".to_string());
+    let db_dir = path.join("db").join("sequence".to_string());
     EventStore::new(id, db_dir.as_path()).map_err(Error::from)
 }
 
