@@ -10,25 +10,19 @@
 
 mod capacity;
 mod chaos;
-mod chunks;
+mod chunk_store;
 /// Configuration handling
 pub mod config_handler;
-mod data_store;
 mod error;
 mod event_mapping;
 mod metadata;
 mod network;
 mod node_api;
 mod node_ops;
-mod section_funds;
-mod to_db_key;
-mod transfers;
 
 /// Docs
 pub mod state_db;
 
-pub mod utils;
-
 pub use crate::node::{
     config_handler::{add_connection_info, set_connection_info, Config},
     error::{Error, Result},
