@@ -13,10 +13,10 @@ mod messaging;
 mod role;
 mod split;
 
+use crate::dbs::UsedSpace;
 use crate::messaging::client::ClientMsg;
 use crate::node::{
-    chunks::Chunks,
-    data_store::UsedSpace,
+    chunk_store::ChunkStore,
     error::convert_to_error_message,
     event_mapping::{map_routing_event, Mapping, MsgContext},
     network::Network,
@@ -67,7 +67,7 @@ pub struct Node {
     network_api: Network,
     node_info: NodeInfo,
     used_space: UsedSpace,
-    role: Role,
+    role: Arc<RwLock<Role>>,
 }
 
 impl Node {
@@ -99,23 +99,19 @@ impl Node {
             reward_key,
         };
 
+        let used_space = UsedSpace::new(config.max_capacity());
+
         let node = Self {
-            role: Role::Adult(AdultRole {
-                chunks: Arc::new(RwLock::new(
-                    Chunks::new(node_info.root_dir.as_path(), config.max_capacity()).await?,
-                )),
-            }),
+            role: Arc::new(RwLock::new(Role::Adult(AdultRole {
+                chunks: Arc::new(
+                    ChunkStore::new(node_info.root_dir.as_path(), used_space.clone()).await?,
+                ),
+            }))),
             node_info,
-            used_space: UsedSpace::new(config.max_capacity()),
+            used_space,
             network_api: network_api.clone(),
         };
 
-        messaging::send(
-            Self::register_wallet(&network_api, reward_key).await,
-            &node.network_api,
-        )
-        .await?;
-
         let our_pid = std::process::id();
         let node_prefix = node.our_prefix().await;
         let node_name = node.our_name().await;
@@ -165,7 +161,7 @@ impl Node {
     /// Starts the node, and runs the main event loop.
     /// Blocks until the node is terminated, which is done
     /// by client sending in a `Command` to free it.
-    pub async fn run(&mut self, network_events: EventStream) -> Result<()> {
+    pub async fn run(&self, network_events: EventStream) -> Result<()> {
         let network_api = self.network_api.clone();
         let event_lock = Arc::new(Mutex::new(network_events));
         let routing_task_handle = tokio::spawn(Self::process_routing_event(
@@ -207,7 +203,7 @@ impl Node {
     }
 
     fn handle_and_get_threads(
-        &mut self,
+        &self,
         op: NodeDuty,
         ctx: Option<MsgContext>,
     ) -> BoxFuture<Vec<JoinHandle<Result<NodeTask>>>> {
